// bitbang.s
// (c) 2009 Matthias Reichl <hias@horus.com>
//
// serial output using bit-banging

#include <avr/io.h>

.global USART_Transmit_Byte

.func USART_Transmit_Byte
USART_Transmit_Byte:

; r26 is used as a bit counter
	ldi r26,8

; transmit low start bit
	cbi _SFR_IO_ADDR(PORTD), PIND1
	lds r25,start_bit_delay
	rcall DelayAtariX	; 8*r25

bitlp:	ror r24		; 1

	brcc bit0	; 1/2

; transmit high bit
	sbi _SFR_IO_ADDR(PORTD), PIND1 ; 2
	rjmp bitdel	; 2

; transmit low bit
bit0:	cbi _SFR_IO_ADDR(PORTD), PIND1 ; 2
	nop		; 1

bitdel:
; 6 cycles per bit so far

	lds r25,bit_delay	; 2
	rcall DelayAtariX	; 8*r25

	nop			; 1
	nop			; 1
	nop			; 1
	nop			; 1
	nop			; 1

; Hias: add one additional Atmel cycle so that transmission
; at pokey divisor 0 matches PAL rate (113 cycles per bit instead of 112)
	nop

	dec r26			; 1
	brne bitlp		; 1/2
	nop			; 1

; total: 17 + r25*8 cycles per data bit


; transmit high stop bit
	sbi _SFR_IO_ADDR(PORTD), PIND1 ; 2
	lds r25,bit_delay
	rcall DelayAtariX

	ret

// delay for r25 Atari clock cycles (8*r25 Atmel cycles)
DelayAtariX:
;	rcall DelayAtariX	// 3
		dec r25		// 1
delata1:	nop		// 1
		nop		// 1
		nop		// 1
		nop		// 1
		nop		// 1
		dec r25		// 1
		brne delata1	// 1/2
		nop		// 1
		ret		// 4
.endfunc
